from setuptools import setup

setup(
    name='SWaN_pack',
    version='1.0',
    packages=[''],
    url='',
    license='',
    author='binodtc',
    author_email='',
    description='Classification of 24-hour accelerometer dataset into wake-wear, sleep-wear, and non-wear'
)
